# Contributing to `pbest`

Contributions are welcome, and they are greatly appreciated!
Every little bit helps, and here's what you need to know:

## Contribution Agreement
Review the [Contributor License Agreement](CLA.md) (CLA). Whenever
you submit a contribution to pbest, you are agreeing to the
CLA, which grants us rights to use your contribution. You don't need
to do anything special to "sign" the CLA. Instead, opening a pull
request indicates your agreement to the CLA.

## Code of Conduct
We expect kindness and respect in all interactions with the
BioSimulators community. Harassing or insulting behavior is
not welcome and will result in enforcement actions.
Please review our [code of conduct](CODE_OF_CONDUCT.md), which describes
our expectations and enforcement actions in more detail.

## Contacting Us
To contact the maintainers privately (for example to report a
violation of our [code of conduct](CODE_OF_CONDUCT.md) or to notify us
of a security issue), you can use the contact information in the
[`AUTHORS.md` file](AUTHORS.md).

## Types of Contributions
You can contribute in many ways:

### Report Bugs

Report bugs at https://github.com/biosimulations/biosim-registry/issues

If you are reporting a bug, please include:

- Your operating system name and version.
- Any details about your local setup that might be helpful in troubleshooting.
- Detailed steps to reproduce the bug.

### Fix Bugs

Look through the GitHub issues for bugs.
Anything tagged with "bug" and "help wanted" is open to whoever wants to implement a fix for it.

### Implement Features

Look through the GitHub issues for features.
Anything tagged with "enhancement" and "help wanted" is open to whoever wants to implement it.

### Write Documentation

pbest could always use more documentation, whether as part of the official docs, in docstrings, or even on the web in blog posts, articles, and such.

### Submit Feedback

The best way to send feedback is to file an issue at https://github.com/biosimulations/biosim-registry/issues.

If you are proposing a new feature:

- Explain in detail how it would work.
- Keep the scope as narrow as possible, to make it easier to implement.
- Remember that this is a volunteer-driven project, and that contributions
  are welcome :)

## Get Started!

Ready to contribute? Here's how to set up `pbest` for local development.
Please note this documentation assumes you already have `uv` and `Git` installed and ready to go.

1. Fork the `pbest` repo on GitHub.

2. Clone your fork locally:

```bash
cd <directory_in_which_repo_should_be_created>
git clone git@github.com:YOUR_NAME/bsedic.git
```

3. Now we need to install the environment. Navigate into the directory

```bash
cd pbest
```

Then, install and activate the environment with:

```bash
uv sync
```

4. Install pre-commit to run linters/formatters at commit time:

```bash
uv run pre-commit install
```

5. Create a branch for local development:

```bash
git checkout -b name-of-your-bugfix-or-feature
```

Now you can make your changes locally.

6. Don't forget to add test cases for your added functionality to the `tests` directory.

7. When you're done making changes, check that your changes pass the formatting tests.

```bash
make check
```

Now, validate that all unit tests are passing:

```bash
make test
```

9. Before raising a pull request you should also run tox.
   This will run the tests across different versions of Python:

```bash
tox
```

This requires you to have multiple versions of python installed.
This step is also triggered in the CI/CD pipeline, so you could also choose to skip this step locally.

10. Commit your changes and push your branch to GitHub:

```bash
git add .
git commit -m "Your detailed description of your changes."
git push origin name-of-your-bugfix-or-feature
```

11. Submit a pull request through the GitHub website.
Before you submit a pull request, check that it meets these guidelines:

- The pull request should include tests.
- If the pull request adds functionality, the docs should be updated.
   Put your new functionality into a function with a docstring, and add the feature to the list in `README.md`.
